@@ -1,25 +1,25 @@
+""" The basic Service class upon which all the specific services are built. """
 import logging
 from urllib.parse import urlparse
 
 from cryptojwt.jwt import JWT
+from oidcmsg.message import Message
+from oidcmsg.oauth2 import ResponseMessage
+from oidcmsg.oauth2 import is_error_message
 
+from oidcservice import util
 from oidcservice.client_auth import factory as ca_factory
 from oidcservice.exception import ResponseError
 from oidcservice.state_interface import StateInterface
-from oidcservice.util import get_http_body
-from oidcservice.util import get_http_url
 from oidcservice.util import JOSE_ENCODED
 from oidcservice.util import JSON_ENCODED
 from oidcservice.util import URL_ENCODED
-
-from oidcmsg.oauth2 import is_error_message
-from oidcmsg.oauth2 import ResponseMessage
-from oidcmsg.message import Message
-
+from oidcservice.util import get_http_body
+from oidcservice.util import get_http_url
 
 __author__ = 'Roland Hedberg'
 
-logger = logging.getLogger(__name__)
+LOGGER = logging.getLogger(__name__)
 
 SUCCESSFUL = [200, 201, 202, 203, 204, 205, 206]
 
@@ -29,6 +29,7 @@ REQUEST_INFO = 'Doing request with: URL:{}, method:{}, data:{}, https_args:{}'
 
 
 class Service(StateInterface):
+    """The basic Service class."""
     msg_type = Message
     response_cls = Message
     error_msg = ResponseMessage
@@ -41,14 +42,14 @@ class Service(StateInterface):
     request_body_type = 'urlencoded'
     response_body_type = 'json'
 
-    def __init__(self, service_context, state_db, conf=None,
+    def __init__(self, service_context, conf=None,
                  client_authn_factory=None, **kwargs):
-        StateInterface.__init__(self, state_db)
+        StateInterface.__init__(self, service_context.state_db)
 
         if client_authn_factory is None:
-            self.client_authn_factory=ca_factory
+            self.client_authn_factory = ca_factory
         else:
-            self.client_authn_factory=client_authn_factory
+            self.client_authn_factory = client_authn_factory
 
         self.service_context = service_context
         self.default_request_args = {}
@@ -83,13 +84,17 @@ class Service(StateInterface):
         # 1. A keyword argument
         # 2. configured set of default attribute values
         # 3. default attribute values defined in the OIDC standard document
-        for prop in self.msg_type.c_param.keys():
+        for prop in self.msg_type.c_param:
             if prop in ar_args:
                 continue
-            else:
-                try:
-                    ar_args[prop] = getattr(self.service_context, prop)
-                except AttributeError:
+
+            try:
+                ar_args[prop] = getattr(self.service_context, prop)
+            except AttributeError:
+                val = self.service_context.get(prop)
+                if val:
+                    ar_args[prop] = val
+                else:
                     try:
                         ar_args[prop] = self.conf['request_args'][prop]
                     except KeyError:
@@ -132,8 +137,10 @@ class Service(StateInterface):
         _args = self.method_args('pre_construct', **kwargs)
         post_args = {}
         for meth in self.pre_construct:
-            request_args, _post_args = meth(request_args, service=self, **_args)
-            post_args.update(_post_args)
+            request_args, _post_args = meth(request_args, service=self, post_args=post_args,
+                                            **_args)
+            # Not necessarily independent
+            # post_args.update(_post_args)
 
         return request_args, post_args
 
@@ -152,15 +159,14 @@ class Service(StateInterface):
 
         return request_args
 
-    def update_service_context(self, resp, state='', **kwargs):
+    def update_service_context(self, resp, key='', **kwargs):
         """
         A method run after the response has been parsed and verified.
 
         :param resp: The response as a :py:class:`oidcmsg.Message` instance
-        :param state: state value
+        :param key: The key under which the response should be stored
         :param kwargs: Extra key word arguments
         """
-        pass
 
     def construct(self, request_args=None, **kwargs):
         """
@@ -215,11 +221,11 @@ class Service(StateInterface):
             http_args = {}
 
         if authn_method:
-            logger.debug('Client authn method: {}'.format(authn_method))
+            LOGGER.debug('Client authn method: %s', authn_method)
             return self.client_authn_factory(authn_method).construct(
                 request, self, http_args=http_args, **kwargs)
-        else:
-            return http_args
+
+        return http_args
 
     def construct_request(self, request_args=None, **kwargs):
         """
@@ -227,7 +233,7 @@ class Service(StateInterface):
         The request information is gathered and the where and how of sending the
         request is decided.
 
-        :param request_args: Initial request arguments
+        :param request_args: Initial request arguments as a dictionary
         :param kwargs: Extra keyword arguments
         :return: A dictionary with the keys 'url' and possibly 'body', 'kwargs',
             'request' and 'ht_args'.
@@ -235,10 +241,6 @@ class Service(StateInterface):
         if request_args is None:
             request_args = {}
 
-        # remove arguments that should not be included in the request
-        # _args = dict(
-        #    [(k, v) for k, v in kwargs.items() if v and k not in SPECIAL_ARGS])
-
         return self.construct(request_args, **kwargs)
 
     def get_endpoint(self):
@@ -249,8 +251,8 @@ class Service(StateInterface):
         """
         if self.endpoint:
             return self.endpoint
-        else:
-            return self.service_context.provider_info[self.endpoint_name]
+
+        return self.service_context.get('provider_info')[self.endpoint_name]
 
     def get_authn_header(self, request, authn_method, **kwargs):
         """
@@ -283,9 +285,8 @@ class Service(StateInterface):
         """
         return self.default_authn_method
 
-    def get_request_parameters(self, request_body_type="", method="",
-                               authn_method='', request_args=None,
-                               http_args=None, **kwargs):
+    def get_request_parameters(self, request_args=None, method="",
+                               request_body_type="", authn_method='', **kwargs):
         """
         Builds the request message and constructs the HTTP headers.
 
@@ -302,7 +303,6 @@ class Service(StateInterface):
         :param method: HTTP method used.
         :param authn_method: Client authentication method
         :param request_args: Message arguments
-        :param http_args: Initial HTTP header arguments
         :param kwargs: extra keyword arguments
         :return: Dictionary with the necessary information for the HTTP
             request
@@ -315,12 +315,12 @@ class Service(StateInterface):
             request_body_type = self.request_body_type
 
         request = self.construct_request(request_args=request_args, **kwargs)
-
+        LOGGER.debug("Request: %s", request)
         _info = {'method': method}
 
         _args = kwargs.copy()
-        if self.service_context.issuer:
-            _args['iss'] = self.service_context.issuer
+        if self.service_context.get('issuer'):
+            _args['iss'] = self.service_context.get('issuer')
 
         # Client authentication by usage of the Authorization HTTP header
         # or by modifying the request object
@@ -337,11 +337,11 @@ class Service(StateInterface):
         _info['url'] = get_http_url(endpoint_url, request, method=method)
 
         # If there is to be a body part
-        if method  == 'POST':
+        if method == 'POST':
             # How should it be serialized
             if request_body_type == 'urlencoded':
                 content_type = URL_ENCODED
-            elif request_body_type in ['jws','jwe', 'jose']:
+            elif request_body_type in ['jws', 'jwe', 'jose']:
                 content_type = JOSE_ENCODED
             else:  # request_body_type == 'json'
                 content_type = JSON_ENCODED
@@ -367,12 +367,11 @@ class Service(StateInterface):
         # If info is a whole URL pick out the query or fragment part
         if '?' in info or '#' in info:
             parts = urlparse(info)
-            scheme, netloc, path, params, query, fragment = parts[:6]
             # either query of fragment
-            if query:
-                info = query
+            if parts.query:
+                info = parts.query
             else:
-                info = fragment
+                info = parts.fragment
         return info
 
     def post_parse_response(self, response, **kwargs):
@@ -393,12 +392,50 @@ class Service(StateInterface):
         :return: dictionary with arguments to the verify call
         """
 
-        kwargs = {'client_id': self.service_context.client_id,
-                  'iss': self.service_context.issuer,
-                  'keyjar': self.service_context.keyjar,
-                  'verify': True}
+        kwargs = {
+            'client_id': self.service_context.get('client_id'),
+            'iss': self.service_context.get('issuer'),
+            'keyjar': self.service_context.keyjar,
+            'verify': True
+        }
+
+        if self.service_name == "provider_info":
+            if self.service_context.get('issuer').startswith("http://"):
+                kwargs["allow_http"] = True
+
         return kwargs
 
+    def _do_jwt(self, info):
+        args = {'allowed_sign_algs': self.service_context.get_sign_alg(self.service_name)}
+        enc_algs = self.service_context.get_enc_alg_enc(self.service_name)
+        args['allowed_enc_algs'] = enc_algs['alg']
+        args['allowed_enc_encs'] = enc_algs['enc']
+        _jwt = JWT(key_jar=self.service_context.keyjar, **args)
+        _jwt.iss = self.service_context.get('client_id')
+        return _jwt.unpack(info)
+
+    def _do_response(self, info, sformat, **kwargs):
+        try:
+            resp = self.response_cls().deserialize(
+                info, sformat, iss=self.service_context.get('issuer'), **kwargs)
+        except Exception as err:
+            resp = None
+            if sformat == 'json':
+                # Could be JWS or JWE but wrongly tagged
+                # Adding issuer is just a fail-safe. If one things was wrong
+                # then two can be.
+                try:
+                    resp = self.response_cls().deserialize(
+                        info, 'jwt', iss=self.service_context.get('issuer'),
+                        **kwargs)
+                except Exception:
+                    pass
+
+            if resp is None:
+                LOGGER.error('Error while deserializing: %s', err)
+                raise
+        return resp
+
     def parse_response(self, info, sformat="", state="", **kwargs):
         """
         This the start of a pipeline that will:
@@ -422,13 +459,13 @@ class Service(StateInterface):
         if not sformat:
             sformat = self.response_body_type
 
-        logger.debug('response format: {}'.format(sformat))
+        LOGGER.debug('response format: %s', sformat)
 
-        if sformat in ['jose','jws','jwe']:
+        if sformat in ['jose', 'jws', 'jwe']:
             resp = self.post_parse_response(info, state=state)
 
             if not resp:
-                logger.error('Missing or faulty response')
+                LOGGER.error('Missing or faulty response')
                 raise ResponseError("Missing or faulty response")
 
             return resp
@@ -439,58 +476,34 @@ class Service(StateInterface):
             info = self.get_urlinfo(info)
 
         if sformat == 'jwt':
-            args = {'allowed_sign_algs':
-                        self.service_context.get_sign_alg(self.service_name)}
-            enc_algs = self.service_context.get_enc_alg_enc(self.service_name)
-            args['allowed_enc_algs'] = enc_algs['alg']
-            args['allowed_enc_encs'] = enc_algs['enc']
-            _jwt = JWT(key_jar=self.service_context.keyjar, **args)
-            _jwt.iss = self.service_context.client_id
-            info = _jwt.unpack(info)
+            info = self._do_jwt(info)
             sformat = "dict"
 
-        logger.debug('response_cls: {}'.format(self.response_cls.__name__))
-        try:
-            resp = self.response_cls().deserialize(
-                info, sformat, iss=self.service_context.issuer, **kwargs)
-        except Exception as err:
-            resp = None
-            if sformat == 'json':
-                # Could be JWS or JWE but wrongly tagged
-                # Adding issuer is just a fail-safe. If one things was wrong
-                # then two can be.
-                try:
-                    resp = self.response_cls().deserialize(
-                        info, 'jwt', iss=self.service_context.issuer, **kwargs)
-                except Exception as err2:
-                    pass
+        LOGGER.debug('response_cls: %s', self.response_cls.__name__)
 
-            if resp is None:
-                logger.error('Error while deserializing: {}'.format(err))
-                raise
+        resp = self._do_response(info, sformat, **kwargs)
 
-        msg = 'Initial response parsing => "{}"'
-        logger.debug(msg.format(resp.to_dict()))
+        LOGGER.debug('Initial response parsing => "%s"', resp.to_dict())
 
         # is this an error message
         if is_error_message(resp):
-            logger.debug('Error response: {}'.format(resp))
+            LOGGER.debug('Error response: %s', resp)
         else:
             vargs = self.gather_verify_arguments()
-            logger.debug("Verify response with {}".format(vargs))
+            LOGGER.debug("Verify response with %s", vargs)
             try:
                 # verify the message. If something is wrong an exception is
                 # thrown
                 resp.verify(**vargs)
             except Exception as err:
-                logger.error(
-                    'Got exception while verifying response: {}'.format(err))
+                LOGGER.error(
+                    'Got exception while verifying response: %s', err)
                 raise
 
             resp = self.post_parse_response(resp, state=state)
 
         if not resp:
-            logger.error('Missing or faulty response')
+            LOGGER.error('Missing or faulty response')
             raise ResponseError("Missing or faulty response")
 
         return resp
@@ -507,27 +520,33 @@ class Service(StateInterface):
         """
         if attr in self.conf:
             return self.conf[attr]
-        else:
-            return default
+
+        return default
 
 
-def build_services(service_definitions, service_factory, module_dirs,
-                   service_context, state_db, client_authn_factory=None):
+def gather_constructors(service_methods, construct):
+    """Loads the construct methods that are defined."""
+    try:
+        _methods = service_methods
+    except KeyError:
+        pass
+    else:
+        for meth in _methods:
+            try:
+                func = meth['function']
+            except KeyError:
+                pass
+            else:
+                construct.append(util.importer(func))
+
+
+def init_services(service_definitions, service_context, client_authn_factory=None):
     """
-    This function will build a number of :py:class:`oidcservice.service.Service`
-    instances based on the service definitions provided.
+    Initiates a set of services
 
-    :param service_definitions: A dictionary of service definitions. The keys
-        are the names of the subclasses. The values are configurations.
-    :param service_factory: A factory that can initiate a service class
-    :param module_dirs: The directories in which to search for service modules.
-        Directory names are expected to be relative to the oidcservice package.
-        e.g. ['oidc', 'oauth2]. The directories will be search in order until
-        a matching module is found.
+    :param service_definitions: A dictionary containing service definitions
     :param service_context: A reference to the service context, this is the same
         for all service instances.
-    :param state_db: A reference to the state database. Shared by all the
-        services.
     :param client_authn_factory: A list of methods the services can use to
         authenticate the client to a service.
     :return: A dictionary, with service name as key and the service instance as
@@ -535,12 +554,26 @@ def build_services(service_definitions, service_factory, module_dirs,
     """
     service = {}
     for service_name, service_configuration in service_definitions.items():
-        _srv = service_factory(service_name,
-                               module_dirs,
-                               service_context=service_context,
-                               state_db=state_db,
-                               client_authn_factory=client_authn_factory,
-                               conf=service_configuration)
+        try:
+            kwargs = service_configuration['kwargs']
+        except KeyError:
+            kwargs = {}
+
+        kwargs.update({
+                          'service_context': service_context,
+                          'client_authn_factory': client_authn_factory
+                      })
+
+        if isinstance(service_configuration['class'], str):
+            _srv = util.importer(service_configuration['class'])(**kwargs)
+        else:
+            _srv = service_configuration['class'](**kwargs)
+
+        if 'post_functions' in service_configuration:
+            gather_constructors(service_configuration['post_functions'], _srv.post_construct)
+        if 'pre_functions' in service_configuration:
+            gather_constructors(service_configuration['pre_functions'], _srv.pre_construct)
+
         try:
             service[_srv.service_name] = _srv
         except AttributeError:
